@@ -2,7 +2,7 @@ case $XARCH in
 i386)
   uname -a
 
-  git clone git://github.com/ocaml/ocaml -b trunk --depth 1
+  git clone git://github.com/ocaml/ocaml -b $OCAML_BRANCH --depth 1
   cd ocaml
   ./configure
   make world.opt
@@ -17,7 +17,7 @@ i386)
   cd ..
   rm -rf ocamlbuild
 
-  ./configure && make && sudo make install
+  ./configure $CONFIGURE_ARGS && make && sudo make install
   ;;
 *)
   echo unknown arch
